/*!
  \file Calls.cpp

  \brief Call-related stuff, definations

  $Id $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   19.08.2005 13:48:29 MSD
*/

#include "h/Calls.h"

using namespace Ice;

CallAttrsDesc_t::AttrsDescription()
{
  Add(clTimeStamp, 		"TIMESTAMP");
  Add(clPartnerHandle,		"PARTNER_HANDLE");
  Add(clPartnerDispname,	"PARTNER_DISPNAME");
  Add(clConfId,			"CONF_ID");
  Add(clType,			"TYPE");
  Add(clStatus,			"STATUS");
  Add(clFailureReason,		"FAILUREREASON");
  Add(clSubject,		"SUBJECT");
  Add(clPSTNStatus,		"PSTN_STATUS");
  Add(clDuration,		"DURATION");
  Add(clPSTNNumber,		"PSTN_NUMBER");
  Add(clConfParticipantsCount,	"CONF_PARTICIPANTS_COUNT");
  Add(clConfParticipant,	"CONF_PARTICIPANT");
  Add(clVMDuration,		"VM_DURATION");
  Add(clVMAllowedDuration,	"VM_ALLOWED_DURATION");
  Add(clSeen,			"SEEN");
  Add(clDTMF,			"DTMF");
  Add(clJoinConference, 	"JOIN_CONFERENCE");
  Add(clVideoStatus,		"VIDEO_STATUS");
  Add(clVideoSendStatus,	"VIDEO_SEND_STATUS");
  Add(clVideoReceiveStatus,	"VIDEO_RECEIVE_STATUS");
};

Call::Call(SkypeConnection& _sc, const int& _id, const LogFunc_t& _log_func) 
     : 	Parent_t(_sc, *this, "CALL", _id, _log_func)
{
  Init(); 	
};	

Call::Call(SkypeConnection& _sc, const int& _id, const ObjErrHandler_t& _eh, const LogFunc_t& _log_func)
     : Parent_t(_sc, *this, "CALL", _id, _eh, _log_func)
{
  Init();	
};

void Call::Init()
{
  struct Parser_t::SkypeObjRegEx re;
  re.resp = boost::regex("^\\s*CALL\\s+(\\w+)\\s+(\\w+)\\s+(.+)\\s*$");
  re.resp_value_pos = 3;
  re.event_attr_pos = 2;
  re.event_value_pos = 3;
  parser.SetRegEx(re);
};

Call::LiveMode_t Call::IsAlive() const
{
  const_iterator p = find(clStatus);
  if(p == end()) return unknown;

  const char* live[7] = { "ROUTING", "EARLYMEDIA", "RINGING", "INPROGRESS", "LOCALHOLD", "REMOTEHOLD", "ONHOLD" };
  for(int i=0;i<7;i++) if(p->second == live[i]) return alive;
  
  return finished;
};

CallSearchDesc_t::AttrsDescription()
{
  Add(srCalls,			"CALLS");
  Add(srActiveCalls,	"ACTIVECALLS");
  Add(srMissedCalls,	"MISSEDCALLS");
};



CallManager::CallManager(SkypeConnection& _sc, const LogFunc_t& _log_func) 
	    : LogAbility(_log_func), Parent_t(_sc, *this, _log_func), sc(_sc)
{ Init(); };

CallManager::CallManager(SkypeConnection& _sc,  const Call::ObjErrHandler_t& _eh, const LogFunc_t& _log_func)
	    : LogAbility(_log_func), Parent_t(_sc, *this, _eh, _log_func), sc(_sc)
{ Init(); };

void CallManager::Init()
{	
  AddEvent("^\\s*CALL\\s+(\\w+)\\s+(\\w+)\\s+(.+)\\s*$");
};	

int CallManager::ParseEvent(const boost::cmatch& _match)
{
  Assert(_match.size() == 4);
  
  return GetFromString<int>(_match[1].str());	
};	

void CallManager::MakeCall(const std::string& _name)
{
  sc << DisableIds << "CALL " << _name << flush << EnableIds;   	
};	
