/*!
  \file Chat.cpp

  \brief Chat-related stuff, definations

  $Id: Chat.cpp,v 1.14 2006/07/06 07:33:50 tarle Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   19.08.2005 13:48:29 MSD
*/

#include "h/Chat.h"

using namespace Ice;

ChatMsgAttrsDesc_t::AttrsDescription()
{
  Add(cmTimeStamp, 	"TIMESTAMP");
  Add(cmFromHandle,	"FROM_HANDLE");
  Add(cmFromDispName,	"FROM_DISPNAME");
  Add(cmType,		"TYPE");
  Add(cmStatus,		"STATUS", 1, 0);
  Add(cmLeaveReason,	"LEAVEREASON");
  Add(cmBody,		"BODY");
  Add(cmChatName,	"CHATNAME");
  Add(cmUsers,		"USERS");
  Add(cmSeen,		"SEEN", 0, 1);
};

ChatMsg::ChatMsg(SkypeConnection& _sc, const int& _id, const LogFunc_t& _log_func) 
     : 	Parent_t(_sc, *this, "CHATMESSAGE", _id, _log_func)
{
  Init(); 	
};	

ChatMsg::ChatMsg(SkypeConnection& _sc, const int& _id,  const ObjErrHandler_t& _eh, const LogFunc_t& _log_func)
    : Parent_t(_sc, *this, "CHATMESSAGE", _id, _eh, _log_func)
{
  Init();	
};

ChatMsg::LiveMode_t ChatMsg::IsAlive() const
{
  const_iterator p = find(cmStatus);
  if(p == end()) return unknown;

  const char* live[2] = { "SENDING", "RECEIVED" };
  for(int i=0;i<2;i++) if(p->second == live[i]) return alive;
  
  return finished;
};

void ChatMsg::Init()
{
  struct Parser_t::SkypeObjRegEx re;
  re.resp = boost::regex("^\\s*CHATMESSAGE\\s+(\\w+)\\s+(\\w+)\\s+(.*)$");
  re.resp_value_pos = 3;
  re.event_attr_pos = 2;
  re.event_value_pos = 3;
  parser.SetRegEx(re);
};


ChatMsgSearchDesc_t::AttrsDescription()
{
  Add(scmChatMsgs,			"CHATMESSAGES");
  Add(scmMissedChatMsgs,	"MISSEDCHATMESSAGES");
};


ChatMsgManager::ChatMsgManager(SkypeConnection& _sc, const LogFunc_t& _log_func) 
   : LogAbility(_log_func), Parent_t(_sc,*this, _log_func), sc(_sc)
{ Init(); };

ChatMsgManager::ChatMsgManager(SkypeConnection& _sc,  const ChatMsg::ObjErrHandler_t& _eh, const LogFunc_t& _log_func)
	    : LogAbility(_log_func), Parent_t(_sc, *this, _eh, _log_func), sc(_sc)
{ Init(); };

void ChatMsgManager::Init()
{	
  AddEvent("^\\s*CHATMESSAGE\\s+(\\w+)\\s+(\\w+)\\s+(\\w+)\\s*$");
};	

int ChatMsgManager::ParseEvent(const boost::cmatch& _match)
{
  Assert(_match.size() == 4);
  
  return GetFromString<int>(_match[1].str());	
};	

/*
 *                           Chat skype object implementation
*/

ChatAttrsDesc_t::AttrsDescription()
{
  Add(chName, 	            "NAME");//!
  Add(chTimeStamp,	    "TIMESTAMP");//!
  Add(chAdder,	            "ADDER");
  Add(chStatus,		    "STATUS");//!
  Add(chPosters,	    "POSTERS");
  Add(chMembers,	    "MEMBERS");//!
  Add(chTopic,	            "TOPIC");
  Add(chChatMessages,	    "CHATMESSAGES");
  Add(chActiveMembers,	    "ACTIVEMEMBERS");//!
  Add(chFriendlyName,	    "FRIENDLYNAME");//!
  Add(chRecentChatMessages, "RECENTCHATMESSAGES");
};

ChatSearchDesc_t::AttrsDescription()
{
  Add(srChats, "CHATS");
  Add(srActiveChats, "ACTIVECHATS");
  Add(srMissedChats, "MISSEDCHATS");
  Add(srRecentChats, "RECENTCHATS");
  Add(srBookmarkedChats, "BOOKMARKEDCHATS");
};

Chat::Chat(SkypeConnection& _sc, const std::string& _id, const LogFunc_t& _log_func) 
     : 	Parent_t(_sc, *this, "CHAT", _id, _log_func)
{
  Init(); 	
};	

Chat::Chat(SkypeConnection& _sc, const std::string& _id,  const ObjErrHandler_t& _eh, const LogFunc_t& _log_func)
    : Parent_t(_sc, *this, "CHAT", _id, _eh, _log_func)
{
  Init();	
};

Chat::LiveMode_t Chat::IsAlive() const
{
  const_iterator p = find(chStatus);
  if(p == end()) return unknown;

  const char* live[3] = { "LEGACY_DIALOG", "DIALOG", "MULTI_SUBSCRIBED" };
  for(int i = 0; i < 3; i++) if(p->second == live[i]) return alive;
  
  return finished;
};

void Chat::Init()
{
  struct Parser_t::SkypeObjRegEx re;
  re.resp = boost::regex("^\\s*CHAT\\s+([^\\s]+)\\s+(\\w+)\\s+(.*)$");
  re.resp_value_pos = 3;
  re.event_attr_pos = 2;
  re.event_value_pos = 3;
  parser.SetRegEx(re);
};

ChatManager::ChatManager(SkypeConnection& _sc, const LogFunc_t& _log_func) 
   : LogAbility(_log_func), Parent_t(_sc,*this, _log_func), sc(_sc)
{ Init(); };

ChatManager::ChatManager(SkypeConnection& _sc,  const Chat::ObjErrHandler_t& _eh, const LogFunc_t& _log_func)
	    : LogAbility(_log_func), Parent_t(_sc, *this, _eh, _log_func), sc(_sc)
{ Init(); };

void ChatManager::Init()
{	
  AddEvent("^\\s*CHAT\\s+([^\\s]+)\\s+(\\w+)\\s+(.+)\\s*$");
};	

std::string ChatManager::ParseEvent(const boost::cmatch& _match)
{
  Assert(_match.size() == 4);
  
  return _match[1];	
};

void ChatManager::MakeChat(const std::string& _name)
{
  sc << DisableIds << "CHAT CREATE " << _name << flush << EnableIds;   	
};	

