/*!
  \file Privileges.cpp

  \brief User privileges and balance implementation

  $Id: Privileges.cpp,v 1.14 2006/04/17 06:44:11 vit Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   09.08.2005 11:32:26 MSD
*/

#include "h/Privileges.h"

using namespace Ice;

PrivAttrsDesc_t::AttrsDescription()
{
  Add(plSkypeOut,  "SKYPEOUT");
  Add(plSkypeIn,   "SKYPEIN");
  Add(plVoiceMail, "VOICEMAIL");
};	

Privileges::Privileges(SkypeConnection& _sc, const LogFunc_t& _log_func)
	   : Parent_t(_sc, *this, "PRIVILEGE", 0, _log_func), sc(_sc)
{ Init(); };

Privileges::Privileges(SkypeConnection& _sc, const ObjErrHandler_t& _eh, const LogFunc_t& _log_func)
	   : Parent_t(_sc, *this, "PRIVILEGE", 0, _eh, _log_func), sc(_sc)
{ Init(); };

void Privileges::Init()
{	
  const char* re_str = 	"^\\s*PRIVILEGE\\s+(\\w+)\\s+(\\w+)\\s*$";
  
  struct Parser_t::SkypeObjRegEx re;
  re.resp = boost::regex(re_str);
  re.resp_value_pos = 2;
  re.event_attr_pos = 1;
  re.event_value_pos = 2;
  parser.SetRegEx(re);
  
  id = sc.RegisterCallback(re_str, RegExNotify_t(this, &Privileges::ProcessEvent));	
};

Privileges::~Privileges()
{
  sc.UnRegisterCallback("^\\s*PRIVILEGE\\s+(\\w+)\\s+(\\w+)\\s*$", id);
};

ProfileAttrsDesc_t::AttrsDescription()
{
  Add(prPSTNBalance, "PSTN_BALANCE");
  Add(prPSTNBalanceCurr, "PSTN_BALANCE_CURRENCY");
  Add(prFullName, "FULLNAME");
  Add(prBirthday, "BIRTHDAY");
  Add(prSex, "SEX");
  Add(prLanguages, "LANGUAGES");
  Add(prCountry, "COUNTRY");
  Add(prProvince, "PROVINCE");
  Add(prCity, "CITY");
  Add(prPhoneHome, "PHONE_HOME");
  Add(prPhoneOffice, "PHONE_OFFICE");
  Add(prPhoneMobile, "PHONE_MOBILE");
  Add(prHomePage, "HOMEPAGE");
  Add(prAbout, "ABOUT");
  Add(prMoodText, "MOOD_TEXT");
  Add(prTimeZone, "TIMEZONE");
  Add(prCallNoAnswerTimeout, "CALL_NOANSWER_TIMEOUT");
  Add(prCallForwardRules, "CALL_FORWARD_RULES");
};

Profile::Profile(SkypeConnection& _sc, const LogFunc_t& _log_func)
	: Parent_t(_sc, *this, "PROFILE", 0, _log_func), sc(_sc)
{ Init(); };

Profile::Profile(SkypeConnection& _sc, const ObjErrHandler_t& _eh, const LogFunc_t& _log_func)
	: Parent_t(_sc, *this, "PROFILE", 0, _eh, _log_func), sc(_sc)
{ Init(); };

void Profile::Init()
{
  const char* re_str = 	"^\\s*PROFILE\\s+(\\w+)\\s+(.*)\\s*$";
  
  struct Parser_t::SkypeObjRegEx re;
  re.resp = boost::regex(re_str);
  re.resp_value_pos = 2;
  re.event_attr_pos = 1;
  re.event_value_pos = 2;
  parser.SetRegEx(re);
  
  id = sc.RegisterCallback(re_str, RegExNotify_t(this, &Profile::ProcessEvent));	
};

Profile::~Profile()
{
  sc.UnRegisterCallback("^\\s*PROFILE\\s+(\\w+)\\s+(.*)\\s*$", id);
};

