/*!
  \file Privileges.h

  \brief  User privileges and profile 
  
  $Id: Privileges.h,v 1.13 2006/03/27 14:49:48 dimak Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   09.08.2005 10:30:18 MSD
*/

#ifndef PRIVILEGES_H
#define PRIVILEGES_H

#include "SkypeObj.h"

namespace Ice {

/// Possible user privileges
typedef enum {
	plSkypeOut,
	plSkypeIn,
	plVoiceMail
} PrivAttrs_t;

/// Privelege attributes description
typedef AttrsDescription<PrivAttrs_t> PrivAttrsDesc_t;

/// Describes atribute of Skype object "Privelege"
template <>
class AttrsDescription<PrivAttrs_t> : public AttrsDescriptionBase<PrivAttrs_t> {
public:
	/// Fills description
	AttrsDescription();
};

/// This class stores user privileges. 
class Privileges : public SkypeObject<Privileges, PrivAttrs_t, SkypeNoIdObjCmdFormatter> {
public:
	/*!
  	  \brief  Constructs Privileges object with DEFAULT error handler.  
  		  
	  	  Default error handler doesn't handle any error at all.
		  
  	  \param _sc 		Reference to the Skype Connection
	  \param _log_func	Reference to the logger. This logger will be used to log any messages inside the object.
	*/
	Privileges(SkypeConnection& _sc, const LogFunc_t& _log_func = DummyLogger);
	/*!
	  \brief  Constructs Privileges object with NON-DEFAULT (user-supplied) error handler.  

	  \param _sc		Reference to Skype Connection
	  \param _eh		Error hanlder.  This functor will be called in case of errors.
	  \param _log_func	Reference to the logger. This logger will be used to log any messages inside the object.
	*/
	Privileges(SkypeConnection& _sc, const ObjErrHandler_t& _eh, const LogFunc_t& _log_func = DummyLogger);
	/// Unregistrates event handlers associated with the object
	~Privileges();
private:
	/// Object-specific initialization.  Called from constructors 
	void Init();
	/// Identifier of the registered event handler
	unsigned id;
	/// Reference to the Skype Connection
	SkypeConnection& sc;
};

/// User profile attributes
typedef enum {
	prPSTNBalance,
	prPSTNBalanceCurr,
	prFullName,
	prBirthday,
	prSex,
	prLanguages,
	prCountry,
	prProvince,
	prCity,
	prPhoneHome,
	prPhoneOffice,
	prPhoneMobile,
	prHomePage,
	prAbout,
	prMoodText,
	prTimeZone,
	prCallNoAnswerTimeout,
	prCallForwardRules
} ProfileAttrs_t;

/// Profile attributes description
typedef AttrsDescription<ProfileAttrs_t> ProfileAttrsDesc_t;

/// Describes atribute of Skype object "Profile"
template <>
class AttrsDescription<ProfileAttrs_t> : public AttrsDescriptionBase<ProfileAttrs_t> {
public:
	/// Fills description
	AttrsDescription();
};

/// This class stores user profile (PSTN Balance and its currency). 
class Profile : public SkypeObject<Profile, ProfileAttrs_t, SkypeNoIdObjCmdFormatter> {
public:
	/*!
  	  \brief  Constructs Profile object with DEFAULT error handler.  
  		  
	  	  Default error handler doesn't handle any error at all.
		  
  	  \param _sc 		Reference to Skype Connection
	  \param _log_func	Reference to the logger. This logger will be used to log any messages inside the object.
	*/
  	Profile(SkypeConnection& _sc, const LogFunc_t& _log_func = DummyLogger);
	/*!
	  \brief  Constructs Profile object with NON-DEFAULT (user-supplied) error handler.  

	  \param _sc		Reference to Skype Connection
	  \param  _eh		Error hanlder.  This functor will be called in case of errors.
	  \param  _log_func	Reference to the logger. This logger will be used to log any messages inside the object.
	*/
	Profile(SkypeConnection& _sc, const ObjErrHandler_t& _eh, const LogFunc_t& _log_func = DummyLogger);
	/// Unregistrates event handlers associated with the object
	~Profile();
private:
	/// Object-specific initialization.  Called from constructors 
	void Init();
	/// Identifier of the registered event handler
	unsigned id;
	/// Reference to the Skype Connection
	SkypeConnection& sc;
};

}; // namespace Ice
#endif // PRIVILEGES_H
