/*!
  \file Calls.cpp

  \brief  Tests of Call Skype object and CallManager

  $Id: Calls.cpp,v 1.9 2006/06/17 15:18:18 vit Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   12.08.2005 00:08:49 MSD
*/

#include <boost/test/unit_test.hpp>
#include <boost/test/unit_test_suite.hpp>

#include "h/Calls.h"

#include "SkypeTest.h"

using namespace boost::unit_test;
using namespace Ice;

SkypeConnection* SC;

void DefaultCallback(const std::string& s)
{
	fprintf(stderr, "%s: '%s'\n", __FUNCTION__, s.c_str());	
};

int ObjCreatedCntr = 0;
std::string last_call_status;
CallManager* CM;

void CallChanged(Call& call, const CallAttrs_t& attr, const std::string& value);

bool ObjCreatedSlot(Call& call) 
{
	call.GetWatchers().RegisterWatcher(CallWatchers_t::AttrSet_t(), CallChanged, true);

	ObjCreatedCntr++;
	return true;	
};

bool CallChanged_called = false;
void CallChanged(Call& call, const CallAttrs_t& attr, const std::string& value)
{
	CallChanged_called = true;
	CallAttrsDesc_t attrs_desc;
	fprintf(stderr, "New value of call %d attribute '%s'is '%s'\n", call.GetId(), attrs_desc[attr].c_str(), value.c_str());	

	if(attr == clStatus) {
		if(value == "INPROGRESS"){
			fprintf(stderr, "%s: requesting hangup\n", __FUNCTION__);
			call.AskForSetAttr(clStatus, "FINISHED");  
		} else if(value == "FINISHED") {
			CM->ReleaseObj(call);
			CM->ReleaseObj(call);
		};	
		last_call_status = value;	
	};	
};

void CallTest()
{
	SkypeConnection& sc = *SC;
	const boost::posix_time::seconds tm(1);
  
	StateWatchers<Call, CallAttrs_t>::AttrSet_t attrs;
  
	CallManager cm(sc, sc.GetLog());
	CM = &cm;
  
	unsigned ids[2];
	ids[0] = cm.RegisterListener(attrs, ObjCreatedSlot);
	ids[1] = cm.RegisterListener(attrs, ObjCreatedSlot);
	cm.UnRegisterListener(ids[1]);
	BOOST_CHECK_NO_THROW(cm.UnRegisterListener(ids[1]));
	BOOST_CHECK_EQUAL(ObjCreatedCntr, 0);
  
	CallChanged_called = false;
	cm.MakeCall("echo123");
	ProcessSkypeMsgs(sc, tm);
	BOOST_CHECK_EQUAL(ObjCreatedCntr, 1);
  
	for(int i = 0; i < 60; ++i) {
		ProcessSkypeMsgs(sc, tm);
		if((last_call_status == "FINISHED") || (last_call_status == "FAILED"))
			break;
	}
	BOOST_CHECK(CallChanged_called == true);

	cm.Squeeze();
};

bool CallSearchResult_called = false;
std::vector<Call*> search_res;
bool CallSearchResult(const std::vector<Call*> _res)
{
	CallSearchResult_called = true;
	BOOST_CHECK(_res.size() > 0);
	fprintf(stderr, "	- Found %d calls\n", _res.size());
	search_res = _res;
  
	return false;
};

void CallSearchTest()
{
	SkypeConnection& sc = *SC;
	const boost::posix_time::seconds tm(1);

	CallWatchers_t::AttrSet_t attrs;
	attrs.insert(clPartnerHandle);
	attrs.insert(clDuration);
	attrs.insert(clTimeStamp);
	attrs.insert(clStatus);
    
	CallManager cm(sc, sc.GetLog());
	CM = &cm;
  
	CallSearchResult_called = false;
	cm.Search(srCalls, CallSearchResult, attrs, "echo123");
  
	for(int i = 0; i < 40; ++i) {
		ProcessSkypeMsgs(sc, tm);
		if(CallSearchResult_called)
			break;
	};
	BOOST_CHECK(CallSearchResult_called == true);

  
	cm.Squeeze();
};

test_suite* init_unit_test_suite(int argc, char* argv[])
{
    test_suite* test= BOOST_TEST_SUITE("Call and CallManager unit test");
    
    static FileLogger<> fl(stderr, LlDbg);
    static SimpleHandler_t def(DefaultCallback);
    static SkypeConnection scl(argv[0], def, LogFunc_t(&fl, &FileLogger<>::operator()));
    scl.Attach();
    scl << DisableIds << "PROTOCOL 99" << flush << EnableIds;

    SC = &scl;
      
    test->add(BOOST_TEST_CASE(&CallTest));
    test->add(BOOST_TEST_CASE(&CallSearchTest));
    
    return test;
};

