/*!
  \file Chat.cpp

  \brief  Tests of 

  $Id: Chat.cpp,v 1.14 2006/06/17 15:18:20 vit Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   12.08.2005 00:08:49 MSD
*/

#include <boost/test/unit_test.hpp>
#include <boost/test/unit_test_suite.hpp>

#include "h/Chat.h"

#include "SkypeTest.h"

using namespace boost::unit_test;
using namespace Ice;

void DefaultCallback(const std::string& s)
{
	fprintf(stderr, "%s: '%s'\n", __FUNCTION__, s.c_str());	
};

int ObjCreatedCntr = 0;
std::vector<ChatMsgAttrs_t> getted_attrs;

Chat* tested_chat = 0;
SkypeConnection* SC;
ChatMsgManager* CM;

bool ChatCreatedSlot(Chat& chat)
{
	std::string name;
  
	BOOST_CHECK_NO_THROW(name = chat[chMembers]);

	bool r = name.find("echo123") != std::string::npos;
	if(r) tested_chat = &chat;

	return r;
};

template <class T>
bool ObjCreatedSlot(T& _obj) 
{
	fprintf(stderr, "HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH\n");	
	ObjCreatedCntr++;
//  _obj.GetWatchers().RegisterWatcher(watcher);
	return true;	
};

void ChatMsgChanged(ChatMsg& chat, const ChatMsgAttrs_t& attr, const std::string& value)
{
	ChatMsgAttrsDesc_t attrs_desc;
	fprintf(stderr, "New value of ChatMsg %d attribute '%s'is '%s'\n", chat.GetId(), attrs_desc[attr].c_str(), value.c_str());	
};

bool ChatSearchResult_called = false;
bool ChatSearchResult(const std::vector<Chat*> _res)
{
	ChatSearchResult_called = true;
	fprintf(stderr, "Executing %s\n", __FUNCTION__);	
	typedef std::vector<Chat*>::const_iterator CI;

	for(CI p=_res.begin();p!=_res.end();++p) {
		const Chat& ch = *(*p);  
//	fprintf(stderr, "%s: '%s'\n", __PRETTY_FUNCTION__, ch[chFriendlyName].c_str());  
	};

	return false;
};

void ChatTest()
{
	fprintf(stderr, "--- ChatTest ---\n");
	SkypeConnection& sc = *SC;
	ChatManager cm(sc, sc.GetLog());
 
	ChatWatchers_t::AttrSet_t attrs;
	attrs.insert(chMembers);

	unsigned ids[2];
	ids[0] = cm.RegisterListener(attrs, ChatCreatedSlot);
	ids[1] = cm.RegisterListener(attrs, ChatCreatedSlot);
	cm.UnRegisterListener(ids[1]);
	BOOST_CHECK_NO_THROW(cm.UnRegisterListener(ids[1]));
	BOOST_CHECK_EQUAL(ObjCreatedCntr, 0);

	cm.MakeChat("echo123");
 
	const boost::posix_time::seconds tm(1);
	ProcessSkypeMsgs(sc, tm);
	BOOST_REQUIRE(tested_chat != 0);

 
	attrs.insert(chTopic);
	attrs.insert(chFriendlyName);

	ChatSearchResult_called = false;
	cm.Search(srChats, ChatSearchResult, attrs);
	for(int i = 0; i < 40; ++i) {
		ProcessSkypeMsgs(sc, tm);
		if(ChatSearchResult_called) 
			break;
	};
	BOOST_CHECK(ChatSearchResult_called == true);
  
	ChatSearchResult_called = false;
	cm.Search(srRecentChats, ChatSearchResult, attrs);
	for(int i = 0; i < 40; ++i) {
		ProcessSkypeMsgs(sc, tm);
		if(ChatSearchResult_called) 
			break;
	};
	BOOST_CHECK(ChatSearchResult_called == true);
 
	ChatSearchResult_called = false;
	cm.Search(srChats, ChatSearchResult, attrs);
	for(int i = 0; i < 40; ++i) {
		ProcessSkypeMsgs(sc, tm);
		if(ChatSearchResult_called) 
			break;
	};
	BOOST_CHECK(ChatSearchResult_called == true);

 
	cm.Squeeze();
};

void ChatMsgTest()
{
	SkypeConnection& sc = *SC;
  
	ChatMsgManager cm(sc, sc.GetLog());
	CM = &cm;
  
	unsigned ids[2];
	ChatMsgWatchers_t::AttrSet_t attrs;
	attrs.insert(cmStatus);
	attrs.insert(cmFromHandle);
	attrs.insert(cmBody);
	  
	ids[0] = cm.RegisterListener(attrs, ObjCreatedSlot<ChatMsg>);
	ids[1] = cm.RegisterListener(attrs, ObjCreatedSlot<ChatMsg>);
	cm.UnRegisterListener(ids[1]);
	BOOST_CHECK_NO_THROW(cm.UnRegisterListener(ids[1]));
	BOOST_CHECK_EQUAL(ObjCreatedCntr, 0);

	const boost::posix_time::seconds tm(1);
	do {
		ProcessSkypeMsgs(sc, tm);
	} while(1);

	cm.Squeeze();
};

bool ChatMsgSearchResult_called = false;
std::vector<ChatMsg*> search_res;
bool ChatMsgSearchResult(const std::vector<ChatMsg*> _res)
{
	ChatMsgSearchResult_called = true;
	BOOST_CHECK(_res.size() > 0);
	fprintf(stderr, "	- Found %d chat messages", _res.size());
	search_res = _res;
  
	return false;
};

void ChatMsgSearchTest()
{
	fprintf(stderr, "--- ChatMsgSearchTest ---\n");
	SkypeConnection& sc = *SC;
  
	ChatMsgManager cm(sc, sc.GetLog());
	CM = &cm;
  
	unsigned ids[2];
	ChatMsgWatchers_t::AttrSet_t attrs;
	attrs.insert(cmStatus);
	attrs.insert(cmFromHandle);
	  

	ChatMsgSearchResult_called = false;
	cm.Search(scmChatMsgs, ChatMsgSearchResult, attrs, "echo123");

	const boost::posix_time::seconds tm(1);
	for(int i = 0; i < 40; ++i) {
		ProcessSkypeMsgs(sc, tm);
		if(ChatMsgSearchResult_called)
			break;
	};
	BOOST_CHECK(ChatMsgSearchResult_called == true);

	cm.Squeeze();
};



test_suite* init_unit_test_suite(int argc, char* argv[])
{
    test_suite* test= BOOST_TEST_SUITE("Chat and ChatMsgManager unit test");
    
    static FileLogger<> fl(stderr, LlDbg);
    static SimpleHandler_t def(DefaultCallback);
    static SkypeConnection scl(argv[0], def, LogFunc_t(&fl, &FileLogger<>::operator()));
    scl.Attach();
    scl << DisableIds << "PROTOCOL 99" << flush << EnableIds;

    SC = &scl;

    test->add(BOOST_TEST_CASE(&ChatTest));
    test->add(BOOST_TEST_CASE(&ChatMsgSearchTest));
    return test;
};











