/*!
  \file SkypeConnection.cpp

  \brief Test of SkypeConection class 

  $Id: SkypeConnection.cpp,v 1.4 2005/10/30 19:30:53 vit Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   27.07.2005 11:56:54 MSD
*/

#include "SkypeTest.h"

#include <boost/test/parameterized_test.hpp>

#include <list>

#include <h/SkypeConnection.h> 

using namespace boost::unit_test;

/// List of called functions
std::list<std::string> called_funcs;
unsigned resp_call_cntr(0);

void SimpleFunc1(const std::string& s)
{
  called_funcs.push_back(__FUNCTION__);	
};

void SimpleFunc2(const std::string& s)
{
  called_funcs.push_back(__FUNCTION__);	
};

void RegExFunc1(const boost::cmatch& m)
{
  called_funcs.push_back(__FUNCTION__);
  BOOST_CHECK_EQUAL(m.size(), 2);
};

void RegExFunc2(const boost::cmatch& m)
{
  called_funcs.push_back(__FUNCTION__);
};

void DefaultCallback(const std::string& s)
{
  BOOST_CHECK(!s.compare("OK"));
  fprintf(stdout, "s = '%s'\n", s.c_str());
  called_funcs.push_back(__FUNCTION__);
};	

void RespHandler(const std::string& s) 
{
  fprintf(stdout, "Executing %s with s = '%s'\n", __FUNCTION__, s.c_str());	
  resp_call_cntr++;
};

void ErrHandler(const int& r, const std::string& e, const std::string& m)
{
  fprintf(stdout, "%s: r = %d, e = '%s', m = '%s'\n", __FUNCTION__, r, e.c_str(), m.c_str());	
};

void SkypeConnectionTst(SkypeConnection* _sc)
{
  const boost::posix_time::seconds tm(1);
  SkypeConnection& sc = *_sc;
  SimpleHandler_t sn(DefaultCallback);
  // Check for exception "Invalid regexp"
  BOOST_CHECK_THROW(sc.RegisterCallback("BAD RE$GEXP(", SimpleHandler_t(DefaultCallback)), AssertionViolation);
  // Check for correct call-back functions execution
  sc.RegisterCallback("^PROTOCOL\\s+(\\d+)\\s*$", RegExNotify_t(RegExFunc1));
  sc.RegisterCallback("^PROTOCOL\\s+(\\d+)\\s*$", RegExNotify_t(RegExFunc2));
  sc.RegisterCallback("^PROTOCOL\\s+(\\d+)\\s*$", SimpleHandler_t(SimpleFunc2));
  sc.RegisterCallback("^PROTOCOL\\s+(\\d+)\\s*$", SimpleHandler_t(SimpleFunc1));

  sc.Attach();
  sc << DisableIds << "PROTOCOL 99" << flush << EnableIds;
  ProcessSkypeMsgs(sc, tm);
  
  std::list<std::string> etalon;
  etalon.push_back("RegExFunc1"); etalon.push_back("RegExFunc2");
  etalon.push_back("SimpleFunc1"); etalon.push_back("SimpleFunc2");
  
  etalon.sort(); called_funcs.sort();
  BOOST_CHECK_EQUAL_COLLECTIONS(called_funcs.begin(), called_funcs.end(), etalon.begin(), etalon.end());
  called_funcs.clear(); etalon.clear();
  
  sc.RegisterCallback("^PONG\\s*$", SimpleHandler_t(SimpleFunc1));
  sc << "PING" << Flush(RespHandler, ErrHandler, 1);
  
  ProcessSkypeMsgs(sc, tm);
  
  etalon.push_back("SimpleFunc1");
  BOOST_CHECK_EQUAL_COLLECTIONS(called_funcs.begin(), called_funcs.end(), etalon.begin(), etalon.end());
  BOOST_CHECK_EQUAL(resp_call_cntr, 1);
  called_funcs.clear(); 
  resp_call_cntr = 0;

  sc << "PING" << Flush(RespHandler, ErrHandler);

  ProcessSkypeMsgs(sc, tm);
  
  BOOST_CHECK_EQUAL_COLLECTIONS(called_funcs.begin(), called_funcs.end(), etalon.begin(), etalon.end());
  BOOST_CHECK_EQUAL(resp_call_cntr, 1);
  called_funcs.clear(); etalon.clear();
  resp_call_cntr = 0;
  
  sc << "PING" << Flush(RespHandler, ErrHandler, 0);

  ProcessSkypeMsgs(sc, tm);
  
  BOOST_CHECK_EQUAL(called_funcs.size(), 0);
  called_funcs.clear(); etalon.clear();
  
  sc << "FOCUS" << flush;
  
  ProcessSkypeMsgs(sc, tm);

  etalon.push_back("DefaultCallback");
  BOOST_CHECK_EQUAL_COLLECTIONS(called_funcs.begin(), called_funcs.end(), etalon.begin(), etalon.end());
  called_funcs.clear(); etalon.clear();
};

test_suite* init_unit_test_suite( int argc, char* argv[] )
{
    test_suite* test= BOOST_TEST_SUITE("SkypeConnection test");
    static std::list<SkypeConnection*> scl;

    static FileLogger<> fl(stderr, LlDbg);
    LogFunc_t log(&fl, &FileLogger<>::operator());
    static SimpleHandler_t def(DefaultCallback);
    static SkypeConnection sc(argv[0], def, log);
    scl.push_back(&sc);
    
    test->add(BOOST_PARAM_TEST_CASE(&SkypeConnectionTst, scl.begin(), scl.end()));
    
    return test;
}

