/*!
  \file win/LowLevel.cpp

  \brief Low-level ++Skype API definations.  Windows implementation.

  $Id: LowLevel.cpp,v 1.23 2006/06/15 09:43:29 tarle Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   19.07.2005 23:29:01 MSD
*/

#include "../h/Cfg.h"

#if defined(SKYPE_MINGW) || defined(SKYPE_MSC)

#include "../h/LowLevel.h"

void* pthis = (void*)-1;

/*!
  I don't understand why should SendMsg("SkypeControlAPIDiscover") twice! %-o
*/
const int __attach_repeat = 2;

/**********************************************************************************************************/
/************************************ Implementation ******************************************************/
/**********************************************************************************************************/
WinRawSkypeConnection_t::RawSkypeConnection(const RawNotify_t& _cbk, const std::string& _app_name, LogFunc_t _log_func) 
	: SkypeCmdFormatter< RawSkypeConnection<os_win> >(*this),
	LogAbility(_log_func),
	cbk(_cbk), app_name(_app_name), attach_result(false), access_denied(false), 
	pending_authorization(false), isInAttach(false)
{
	skypeControlAPIAttachID = (UINT)-1;
	skypeControlAPIDiscoverID = (UINT)-1;
	pthis = this;

	skypeControlAPIAttachID = RegisterWindowMessage("SkypeControlAPIAttach");
	//fprintf(stderr, "skypeControlAPIAttachID = %d\n", skypeControlAPIAttachID);
	
	skypeControlAPIDiscoverID = RegisterWindowMessage("SkypeControlAPIDiscover");
	//fprintf(stderr, "skypeControlAPIDiscoverID = %d\n", skypeControlAPIDiscoverID);

	bool retval = Initialize_CreateWindowClass();
	//fprintf(stderr, "Initialize_CreateWindowClass = %d\n", (int)retval);
	retval = Initialize_CreateMainWindow();
	//fprintf(stderr, "nitialize_CreateMainWindow = %d\n", (int)retval);
};	

void WinRawSkypeConnection_t::Attach(int _timeout)
{
	if(isInAttach)
		return;
	isInAttach = true;
//	fprintf(stderr, "WinRawSkypeConnection_t::Attach\n");
	for(int i = 0; i < __attach_repeat; ++i)
	{
		//fprintf(stderr, "WinRawSkypeConnection_t::Attach SendMessage\n");
		if(PostMessage( 
			HWND_BROADCAST, skypeControlAPIDiscoverID, (WPARAM)mainWindowHandle, 0) == 0)
			throw SkypeDisconnect("Discover SkypeAPI  message could not be sent"); 

		boost::posix_time::ptime _t_beg(boost::posix_time::microsec_clock::universal_time());
		boost::posix_time::ptime _t_cur(boost::posix_time::microsec_clock::universal_time());
		boost::posix_time::time_duration _t_timeout = boost::posix_time::milliseconds(_timeout);

		MSG oMessage;
		while(!(attach_result || access_denied) && (((_t_cur - _t_beg) < _t_timeout) || pending_authorization))
		{
			//fprintf(stderr, "WinRawSkypeConnection_t::Attach while\n");
			//if(pending_authorization)
			//	fprintf(stderr, "WinRawSkypeConnection_t::Attach pending_authorization\n");
				
			_t_cur = boost::posix_time::microsec_clock::universal_time();

//			if(PeekMessage(&oMessage, mainWindowHandle, skypeControlAPIAttachID, skypeControlAPIAttachID, PM_REMOVE) == 0)
			if(PeekMessage(&oMessage, 0, 0, 0, PM_REMOVE) == 0)
			{
				Impl::Sleep(boost::posix_time::milliseconds(10));
				continue;
			}

//			fprintf(stderr, "Peak Msg. HWND = %X\n", oMessage.hwnd);
			bool retval = (TranslateMessage(&oMessage) != 0);
			//fprintf(stderr, "TranslateMessage %d\n", (int)retval);
			retval = (DispatchMessage(&oMessage) != 0);
			//fprintf(stderr, "DispatchMessage %d\n", (int)retval);
		}
	//	log(LlDbg, "%s: attach_result = %d, access_denied = %d\n", __FUNCTION__, attach_result, access_denied);
		if(access_denied)
		{
			isInAttach = false;
			throw SkypeAccessDenied(); 
		}

		if(!attach_result)
		{
			isInAttach = false;
			throw SkypeDisconnect("Attach to SkypeAPI timeout"); 
		}
	}
	isInAttach = false;

};

void WinRawSkypeConnection_t::Detach()
{
	if(!attach_result) return;
	skypeWindowHandle = NULL;
	attach_result = false;
};

void WinRawSkypeConnection_t::flush(const std::string& cmd)
{
	if(!attach_result)
 		throw SkypeDisconnect("SkypeAPI was not attached"); 

	COPYDATASTRUCT oCopyData;
	
	log(LlDbg,"%s: Send to Skype : '%s'", __FUNCTION__, cmd.c_str()); 

	oCopyData.dwData = 0;
	oCopyData.lpData = (void*)cmd.c_str();
	oCopyData.cbData = cmd.length() + 1;
	if(oCopyData.cbData != 1)
	{
		if(SendMessage(skypeWindowHandle, WM_COPYDATA, (WPARAM)mainWindowHandle, (LPARAM)&oCopyData) == FALSE)
		{
//			DWORD er = GetLastError();
			skypeWindowHandle = NULL;
			attach_result = false;
			throw SkypeDisconnect("SkypeAPI disconnect. SendMessage return -1"); 
		}
	}
};

void WinRawSkypeConnection_t::flush()
{
	SkypeCmdFormatter< RawSkypeConnection<os_win> >::flush();
}


bool WinRawSkypeConnection_t::operator()()
{	
	return false;
};


LRESULT APIENTRY WinRawSkypeConnection_t::_WindowProc(
		HWND hWindow, UINT uiMessage, WPARAM uiParam, LPARAM ulParam)
{
	return reinterpret_cast<WinRawSkypeConnection_t*>(pthis)->WindowProc(hWindow, uiMessage, uiParam, ulParam);
}

LRESULT WinRawSkypeConnection_t::WindowProc(
		HWND hWindow, UINT uiMessage, WPARAM uiParam, LPARAM ulParam)
{
	LRESULT lReturnCode(0);
	bool fIssueDefProc(false);

	switch(uiMessage)
	{
	case WM_DESTROY:
		mainWindowHandle = NULL;
		PostQuitMessage(0);
		break;
	case WM_COPYDATA:
		if(skypeWindowHandle == (HWND)uiParam)
		{
			PCOPYDATASTRUCT poCopyData = (PCOPYDATASTRUCT)ulParam;
			std::string s((const char*)poCopyData->lpData);
			try{
				cbk(s);
			}
			catch(std::exception& e)
			{
				fprintf(stderr, "LibSkype: Callback exception: %s\n", e.what());
			}
			lReturnCode = 1;
		}
		break;
	default:
		if(uiMessage == skypeControlAPIAttachID)
		{
			//fprintf(stderr, "WP    attacg \n");
//			attach_result = false;
			switch(ulParam)
			{
			case SKYPECONTROLAPI_ATTACH_SUCCESS:
				log(LlInfo, "SKYPECONTROLAPI_ATTACH_SUCCESS");
				skypeWindowHandle=(HWND)uiParam;
				attach_result = true;
				pending_authorization = false;
				break;
			case SKYPECONTROLAPI_ATTACH_PENDING_AUTHORIZATION:
				log(LlInfo, "SKYPECONTROLAPI_ATTACH_PENDING_AUTHORIZATION\n");
				pending_authorization = true;
				attach_result = false;
				break;
			case SKYPECONTROLAPI_ATTACH_REFUSED:
				log(LlInfo, "SKYPECONTROLAPI_ATTACH_REFUSED\n");
				access_denied = true;
				attach_result = false;
				break;
			case SKYPECONTROLAPI_ATTACH_NOT_AVAILABLE:
				log(LlInfo, "SKYPECONTROLAPI_ATTACH_NOT_AVAILABLE\n");
				attach_result = false;
				break;
			case SKYPECONTROLAPI_ATTACH_API_AVAILABLE:
				log(LlInfo, "SKYPECONTROLAPI_ATTACH_API_AVAILABLE\n");
				break;
			}
			//fprintf(stderr, "WP    attacg \n");
			lReturnCode = 1;
			break;
		}
		fIssueDefProc = true;
		break;
	}
	if(fIssueDefProc)
		lReturnCode = DefWindowProc(hWindow, uiMessage, uiParam, ulParam);

	return(lReturnCode);
}

bool WinRawSkypeConnection_t::Initialize_CreateWindowClass(void)
{
	unsigned char *paucUUIDString;
	RPC_STATUS lUUIDResult;
	bool fReturnStatus;
	UUID oUUID;

	fReturnStatus=false;
	lUUIDResult=UuidCreate(&oUUID);
	processHandle=(HINSTANCE)OpenProcess(PROCESS_DUP_HANDLE, FALSE, GetCurrentProcessId());
	if(processHandle!=NULL && (lUUIDResult==RPC_S_OK || lUUIDResult==RPC_S_UUID_LOCAL_ONLY) )
	{
		if(UuidToString(&oUUID, &paucUUIDString) == RPC_S_OK)
		{
			WNDCLASS oWindowClass;

			strcpy(windowClassName, "IceBrainsSkypeAPI");
			strcat(windowClassName, (char*)paucUUIDString);

			oWindowClass.style=CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS;
			oWindowClass.lpfnWndProc = (WNDPROC)&(WinRawSkypeConnection_t::_WindowProc);
			oWindowClass.cbClsExtra = 0;
			oWindowClass.cbWndExtra = 0;
			oWindowClass.hInstance = processHandle;
			oWindowClass.hIcon = NULL;
			oWindowClass.hCursor = NULL;
			oWindowClass.hbrBackground = NULL;
			oWindowClass.lpszMenuName = NULL;
			oWindowClass.lpszClassName = windowClassName;

			if(RegisterClass(&oWindowClass)!=0)
				fReturnStatus=true;

			RpcStringFree(&paucUUIDString);
		}
	}
	if(fReturnStatus == false)
		CloseHandle(processHandle),processHandle=NULL;
	return(fReturnStatus);
}

bool WinRawSkypeConnection_t::Initialize_CreateMainWindow(void)
{
	mainWindowHandle=CreateWindowEx(
		WS_EX_APPWINDOW|WS_EX_WINDOWEDGE,
		windowClassName, "", WS_BORDER|WS_SYSMENU|WS_MINIMIZEBOX,
		CW_USEDEFAULT, CW_USEDEFAULT, 128, 128, NULL, 0, processHandle, 0);
	return(mainWindowHandle != NULL ? true : false);
}

#endif //SKYPE_MINGW || SKYPE_MSC
